using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Drawing;

namespace DigiDesign
{
    public class VHDLWriter
    {

        StreamWriter sw = null;

        public string moduleName = "";
        private List<Point> inputPins = new List<Point>();
        private List<Point> outputPins = new List<Point>();
        private int numWires = 0;

        public VHDLWriter(string filename, string modName) {
            sw = new StreamWriter(filename);
            moduleName = modName;
        }

        public void writeHeader() {
            sw.WriteLine("-- Generated by DigiDesign 1.0");
            sw.WriteLine("-- (c)2006 Sushovan De, Tarun Kumar");
            sw.WriteLine("--");
            sw.WriteLine("");
            sw.WriteLine("library IEEE;");
            sw.WriteLine("use IEEE.STD_LOGIC_1164.ALL;");
            sw.WriteLine("use IEEE.STD_LOGIC_ARITH.ALL;");
            sw.WriteLine("use IEEE.STD_LOGIC_UNSIGNED.ALL;");
            sw.WriteLine("");
        }


        public void writeFooter() {
            //
            sw.WriteLine("end structural");
            sw.Flush();
            sw.Close();
        }


        internal void findInputAndOutputNodes(List<DigitalObject> scenegraph) {
            //traverse the scenegraph and find those modules having no earthly inputs
            for (int i = 0; i < scenegraph.Count; i++) {
                DigitalObject dobj = scenegraph[i];
                if (dobj.toolType == ToolboxStrip.DigiTool.Wire) continue;
                for (int j = 0; j < dobj.inputs.Length; j++) {
                    if (dobj.inputs[j] == null) {
                        //then it is an input...
                        inputPins.Add(new Point(i,j));
                        dobj.inputUniqeId.Add(inputPins.Count-1);
                    }
                }
                for (int j = 0; j < dobj.outputs.Length; j++) {
                    if (dobj.outputs[j] == null) {
                        //then it is an input...
                        outputPins.Add(new Point(i, j));
                        dobj.outputUniqeId.Add(outputPins.Count-1);
                    }
                }
            }



        }





        public void writeEntity() {
            sw.WriteLine("");
            sw.WriteLine("ENTITY " + moduleName + " is");
            sw.WriteLine("\tPORT(");

            for (int i = 0; i < inputPins.Count; i++) {
                sw.WriteLine("\t\tinpin" + i + " :IN std_logic");
            }
            for (int i = 0; i < outputPins.Count; i++) {
                sw.WriteLine("\t\toutpin" + i + " :OUT std_logic");
            }

            sw.WriteLine("\t);");
            sw.WriteLine("end " + moduleName);
            sw.WriteLine("");
        }


        public void findNumWires(List<DigitalObject> scenegraph) {
            //simply count the number of wires
            numWires = 0;
            foreach (DigitalObject dobj in scenegraph) {
                if (dobj.toolType == ToolboxStrip.DigiTool.Wire) {
                    dobj.uniqueId = numWires;
                    numWires++;
                }
            }
        }




        internal void writeSignalDeclarations() {
            sw.WriteLine("");
            sw.WriteLine("ARCHITECTURE structural of " + moduleName + " is");
            sw.WriteLine("");
            sw.WriteLine("-- Signal Declarations");
            sw.WriteLine("");
            for (int i = 0; i < numWires; i++) {
                sw.WriteLine("\tsignal wire" + i + " :std_logic");
            }
            sw.WriteLine("");
        }

        internal void writeConnections(List<DigitalObject> scenegraph) {
            //connect the wires.

            sw.WriteLine("begin");
            foreach (DigitalObject dobj in scenegraph) {
                if (dobj.toolType == ToolboxStrip.DigiTool.Wire) {
                    sw.Write("\twire" + dobj.uniqueId + " <= ");
                    DigitalObject prev = dobj.inputs[0];

                    string in1 = "", in2 = "", opsz = "";
                    if (prev.inputs[0] == null) in1 = "inpin" + prev.inputUniqeId[0];
                    else in1 = "wire" + prev.inputs[0].uniqueId;
                    if (prev.inputs.Length > 1) {
                        if (prev.inputs[1] == null) in2 = "inpin" + prev.inputUniqeId[1];
                        else in2 = "wire" + prev.inputs[1].uniqueId;
                    }

                    switch (prev.toolType) {
                        case ToolboxStrip.DigiTool.AndGate:
                            opsz = "and";
                            break;
                        case ToolboxStrip.DigiTool.OrGate:
                            opsz = "or";
                            break;
                        case ToolboxStrip.DigiTool.NotGate:
                            opsz = "not";
                            in2 = in1;
                            in1 = "";//quick hack
                            break;
                        case ToolboxStrip.DigiTool.XorGate:
                            opsz = "xor";
                            break;
                        default:
                            break;
                    }

                    sw.WriteLine(in1 + " " + opsz + " " + in2);

                }


                //now if it is an output..
                if (dobj.outputs[0] == null) {

                    string in1 = "", in2 = "", opsz = "";
                    if (dobj.inputs[0] == null) in1 = "inpin" + dobj.inputUniqeId[0];
                    else in1 = "wire" + dobj.inputs[0].uniqueId;
                    if (dobj.inputs.Length > 1) {
                        if (dobj.inputs[1] == null) in2 = "inpin" + dobj.inputUniqeId[1];
                        else in2 = "wire" + dobj.inputs[1].uniqueId;
                    }

                    switch (dobj.toolType) {
                        case ToolboxStrip.DigiTool.AndGate:
                            opsz = "and";
                            break;
                        case ToolboxStrip.DigiTool.OrGate:
                            opsz = "or";
                            break;
                        case ToolboxStrip.DigiTool.NotGate:
                            opsz = "not";
                            in2 = in1;
                            in1 = "";//quick hack
                            break;
                        case ToolboxStrip.DigiTool.XorGate:
                            opsz = "xor";
                            break;
                        default:
                            break;
                    }

                    sw.WriteLine("\toutpin"+ dobj.outputUniqeId[0]+" <= " +in1 + " " + opsz + " " + in2);
                }
            }
        }




    }
}
